﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.Emit;
using System.Text;

using Xamarin.Forms;

namespace AppBayFinal2019
{
    public class PageBay : ContentPage
    {
        Entry entryTu;
        Entry entryTg;
        public Label labOutT1;
        Label labOutT2;
        Label labOutTd;
        public PageBay()
        {
            // Create the Button and attach Clicked handler. 
            Button button = new Button
            {
                Text = "Berechne Reuter-Identifikation",
                BorderColor = Color.Yellow,
                BackgroundColor = Color.Yellow
            };
            button.Clicked += OnButtonClicked;
            Label LabTitle = new Label
            {
                Text = "App zur Reuteridentifikation",
                FontSize = 20.0
            };
            Label labInTu = new Label
            {
                Text = "Eingabe Tu:",
                FontSize = 15.0
            };
            Label labInTg = new Label
            {
                Text = "Eingabe Tg:",
                FontSize = 15.0
            };
            entryTu = new Entry
            {
                Text = "0",
                Keyboard=Keyboard.Numeric
             };
            entryTg = new Entry
            {
                Text = "0",
                Keyboard = Keyboard.Numeric
            };
            labOutT1 = new Label
            {
                Text = "T1= ",
                FontSize = 15.0
            };
            labOutT2 = new Label
            {
                Text = "T2=",
                FontSize = 15.0
            };
            labOutTd = new Label
            {
                Text = "Td=",
                FontSize = 15.0
            };

            // Assemble the page.
            this.Title = "Reuter- Identifikation";
            this.Content = new StackLayout
            {
                Children =
                {
                    LabTitle,
                    labInTu,
                    entryTu,
                    labInTg,
                    entryTg,
                    button,
                    labOutT1,
                    labOutT2,
                    labOutTd,
                }
            };
        }
        void OnButtonClicked(object sender, EventArgs args)
        {
            double x1 = 0, x2 = 0;
            ConvertToDouble(entryTu.Text, ref x1);
            ConvertToDouble(entryTg.Text, ref x2);
            double t1 = 0, t2 = 0, td = 0;
            Reuterident(x1, x2, ref t1, ref t2, ref td);
            labOutT1.Text = "T1= " + t1.ToString("f6");
            labOutT2.Text = "T2= " + t2.ToString("f6");
            labOutTd.Text = "Td= " + td.ToString("f6");

        }

        //***************************************** externe functions **************************************
        double h2PT1(double t, double T1, double T2)
        {
            double ex1, ex2;
            if (Math.Abs(T1) < 1e-10) T1 = 1e-10;
            if (Math.Abs(T2) < 1e-10) T2 = 1e-10;
            if (t / T1 > 80) ex1 = 0.0; else ex1 = Math.Exp(-t / T1);
            if (t / T2 > 80) ex2 = 0.0; else ex2 = Math.Exp(-t / T2);
            if (Math.Abs(T1 - T2) / T1 > 1e-10) return (1 + (-T1 * ex1 + T2 * ex2) / (T1 - T2));
            else return (1 - (T1 + t) / T1 * ex1);
        }
        //---------------------------------------------------------------------------
        double gradh2pt1(double t, double T1, double T2)
        {
            double ex1, ex2;
            if (t / T1 > 80) ex1 = 0.0; else ex1 = Math.Exp(-t / T1);
            if (t / T2 > 80) ex2 = 0.0; else ex2 = Math.Exp(-t / T2);
            if (Math.Abs(T1 - T2) / T1 > 1e-10)
                return ((ex1 - ex2) / (T1 - T2));
            else return ((t / T1 / T1) * ex1);
        }

        //------------------ ---------------------------------------------------------
        void tutg(ref double tu, ref double tg, double T1, double T2)
        {
            double hw, tw, grad;
            if (Math.Abs(T1 - T2) / T1 > 1e-10)
                tw = T1 * T2 / (T1 - T2) * Math.Log(T1 / T2);
            else tw = T1;
            hw = h2PT1(tw, T1, T2);
            grad = gradh2pt1(tw, T1, T2);
            tg = 1 / grad;
            tu = tw - hw / grad;
        }

        private void Reuterident(double tu, double tg, ref double /*&*/  T1, ref double /*&*/ T2, ref double /*&*/ Tz)
        {
            double x, b, hilf;
            double aa, bb, x1, x2, f1, f2, tu1, tg1;
            double[] powerx;
            int i;

            powerx = new double[7];
            x = tu / tg; Tz = 0;
            if (tu > 0)
            {
                if (x < 0.016)
                {
                    T1 = tg; T2 = tu; b = T2 / T1; hilf = tg / T1;
                }
                else if (x > 0.101)
                {
                    T1 = 0.4649 * tg; T2 = 0.2789 * tg; Tz = tu - tg / 10.0;
                }
                else
                {    /* switch (0.016<x<0.1*/
                    powerx[0] = 1;
                    for (i = 1; i <= 6; i++)
                        powerx[i] = powerx[i - 1] * x;
                    b = 4.718067869278e-2;
                    b = b - 9.221731329728 * powerx[1];
                    b = b + 8.154533741866e2 * powerx[2];
                    b = b - 2.911008862897e4 * powerx[3];
                    b = b + 5.365473286922e5 * powerx[4];
                    b = b - 4.819088731821e6 * powerx[5];
                    b = b + 1.697919973411e7 * powerx[6];
                    hilf = 1.101504362557;
                    hilf = hilf - 1.575911053093e1 * powerx[1];
                    hilf = hilf + 1.596248432828e3 * powerx[2];
                    hilf = hilf - 5.714287489367e4 * powerx[3];
                    hilf = hilf + 1.042091887446e6 * powerx[4];
                    hilf = hilf - 9.231048230465e6 * powerx[5];
                    hilf = hilf + 3.193754111909e7 * powerx[6];
                    T1 = tg / hilf;
                    T2 = T1 * b;
                }
                if (x <= 0.101)// to improve accuracy one step with linear interpolation
                {
                    /*improve T1/T2 */
                    tu1 = 0;
                    tg1 = 0;
                    tutg(ref tu1, ref tg1, T1, T2); x1 = tu1 / tg1; f1 = T2 / T1;   /* 1. point */
                    tutg(ref tu1, ref tg1, T1, T2 * 1.01); x2 = tu1 / tg1; f2 = T2 * 1.01 / T1;  /* 2. point */
                    aa = (f2 - f1) / (x2 - x1); bb = f1 - aa * x1;
                    b = aa * x + bb;
                    /*improve Tg/T1 */
                    tutg(ref tu1, ref tg1, T1, T2); x1 = tu1 / tg1; f1 = tg1 / T1;   /* 1. point */
                    tutg(ref tu1, ref tg1, T1 * 1.01, T2); x2 = tu1 / tg1; f2 = tg1 / T1 / 1.01;  /* 2. point */
                    aa = (f2 - f1) / (x2 - x1); bb = f1 - aa * x1;
                    hilf = aa * x + bb;
                    T1 = tg / hilf;
                    T2 = T1 * b;
                }
            }
        }
        public char comF, comR;   // to correct Comma- / point problem comF : wrong separator
        // comR  : correct separator

        //string wrongDataFormat = "wrong Data Format";


        public void ConvertToDouble(string text, ref double nb)
        {
            // Secure Conversion independant of Komma- Version of PC
            // comF contains String with wrong camma, comR contains string with correct comma
            // if number format is wrong, result is old number
            // Call :  ConvertToDouble(text,ref number);
            // removes all thousand- Seperators
            // examples: (if decimal sep =".")
            // 1000,000,000.23 --> 1000000000.23
            // 1000.000.000,23 --> 1000000000.23
            // 0.23 --> 0.23
            // 0,23 --> 0.23
            // 1000,023 --> 1000.023
            // 1000.023 --> 1000.023

            // examples: (if decimal sep =",")
            // 1000,000,000.23 --> 1000000000,23
            // 1000.000.000,23 --> 1000000000,23
            // 0.23 --> 0,23
            // 0,23 --> 0,23
            // 1000,023 --> 1000,023
            // 1000.023 --> 1000,023

            string TestText = (0.1).ToString("f1");

            if (TestText.Substring(1, 1) == ".")  // Check which is the actual Decimal separator
            { comF = ','; comR = '.'; }
            else
            { comF = '.'; comR = ','; }

            double number = 0;
            char[] seps = new char[2];
            seps[0] = ','; // both possible decimal seperators
            seps[1] = '.';
            // check if Thousand- group sign is available. If more than one sign, all left signs are removed
            int index = 0, cnt = 0;
            while (index > -1)  // count number of seperators
            {
                index = text.IndexOfAny(seps, index);  // look for next position of seps
                if (index > -1) { cnt++; index++; }
            }
            string newtext = text;
            if (cnt > 1)  // If more than one sign, all left signs are removed
            {
                string leftSign = text.Substring(text.IndexOfAny(seps), 1);  // what is the left sign?
                newtext = text.Replace(leftSign, "");// Remove all left signs
            }
            string help = newtext.Replace(comF, comR); // Change into correct decimal sep
            if (Double.TryParse(help, out number))
                nb = number;
            //else MessageBox.Show(wrongDataFormat);

        }
    }
}
